#!/usr/bin/perl -w
# tt-tkdns.pl - This is my attempt at writing a DNS management system. It uses
#		perl\tk, and some of it's modules. It originally came from me 
# 		needing to manage 420+ domains. You may use and modify this as 
#		you wish, I'd just like to know what you did, and why. Maybe
#		it could be included in a later release.
#
# Copyright (C) 2000 Sean Lutner and Adam Glasgall
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distribted in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this programl if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA, 02111-1307, USA

use Tk;

my $ip;
my $hostname;

my $mw = MainWindow->new;
$mw->title("TT-TkDNS");


# This contains a frame which houses the menubar for all of our options.
my $frame = $mw->Frame(-relief => 'raised', -borderwidth => 1);
	$frame->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');

	$frame->Menubutton(-text => "File", -relief => 'flat', -tearoff => 0,
				-menuitems => [[ 'command' => "New(not imp)", -command => \&add_new],
					       [ 'command' => "Edit", -command => \&edit_what],
					       [ 'command' => "Add"],
					       "-",
					       [ 'command' => "Exit", -command => \&close_program]
					      ])->pack(-side => 'left', -expand => 'n', -fill => 'both');

	$frame->Menubutton(-text => "Help", -relief => 'flat', -tearoff => 0,
				-menuitems => [[ 'command' => "About TT-TkDNS", -command => \&about_tt]
				              ])->pack(side => 'right');
# Welcome message for the user
$mw->Label(-width => 80, -text => "Welcome to the TechTarget.com DNS management system.")->pack(-expand => 1, -fill => 'both');
# A frame to display our logo
$logo = $mw->Photo(-file => "resources/tt-logo.gif");
$mw->Label( -image => $logo)->pack(-side => 'bottom');
MainLoop;


# This is the backend of this thing. All the sub routines to read and write to files are
# here. This was the brunt of the work on this thing.

# This sub will do the necessary things to add a new zone file to our records
sub add_new {
	my $newzone = $mw->Toplevel();
	$newzone->title("Add a new zone");

	$zone_frame = $newzone->Frame(-relief => 'flat', borderwidth => 0);
	$zone_frame->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
	$zone_frame->Label(-width => 60, -text => "Please enter the zonefile you wish to create:")->pack(-side => 'left', -expand => 1, -fill => 'both');
	$zone_frame->Entry(-textvariable => \$zone)->pack(-side => 'right');

	$record_frame = $newzone->Frame(-relief => 'flat', borderwidth => 0);
	$record_frame->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
	$record_frame->Label(-width => 60, -text => "Please enter any A records you need:")->pack(-side => 'left', -expand => 1, -fill => 'both');
	$record_frame->Entry(-textvariable => \$arecords)->pack(-side => 'right');

	$cname_frame = $newzone->Frame(-relief => 'flat', borderwidth => 0);
	$cname_frame->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
	$cname_frame->Label(-width => 60, -text => "Please enter any CNAMES you need:")->pack(-side => 'left', -expand => 1, -fill => 'both');
	$cname_frame->Entry(-textvariable => \$cnames)->pack(-side => 'right');

	$ptr_frame = $newzone->Frame(-relief => 'flat', borderwidth => 0);
	$ptr_frame->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
	$ptr_frame->Label(-width => 60, -text => "Please enter and ptrs you may need:")->pack(-side => 'left', -expand => 1, -fill => 'both');
	$ptr_frame->Entry(-textvariable => \$ptrs)->pack(-side => 'right');

	
	$newzone->Button(-text => "Submit New Zone", -command => \&submit_newzone)->pack;
}

# This will edit an existing zonefile or the named.conf
sub edit_what {
	$zonedir = "/var/named";
	$confdir = "/usr/local/etc";
	my $editzone = $mw->Toplevel();
	$editzone->title("Edit a zone or named.conf");

	$zone_edit = $editzone->Frame(-relief => 'flat', borderwidth => 0);
	$zone_edit->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
	$zone_edit->Label(-width => 60, -text => "Please enter the name of the zone file you want to edit:")->pack(-side => 'left', -expand => 1, -fill => 'both');
	$zone_edit->Entry(-textvariable => \$openzone)->pack(-side => 'right');

	$conf_edit = $editzone->Frame(-relief => 'flat', borderwidth => 0);
	$conf_edit->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
	$conf_edit->Label(-width => 60, -text => "Which conf file do you want to edit:")->pack(-side => 'left', -expand => 1, -fill => 'both');
	$conf_edit->Entry(-textvariable => \$openconf)->pack(-side => 'right');
	$editzone->Button(-command => \&editzone, -text => "Edit zone")->pack(-side => 'bottom');
}

# The dialog box for editing a zonefile
sub editzone {
	my $zone_dbox=$mw->Toplevel();
	$zone_dbox->title("Edit Zone");
	$zonep= ($zonedir . "/") . $openzone;
	@hosts=parse();
	my $tbox=$zone_dbox->Scrolled("Text")->pack(-side => 'top');
	$zone_dbox->Label(-text => 'Hostname')->pack(-side => 'left');
	$h_entry = $zone_dbox->Entry(-textvariable => \$hostname)->pack(-side => 'left');
	$zone_dbox->Label(-text => 'IP address')->pack(-side => 'left');
	$i_entry = $zone_dbox->Entry(-textvariable => \$ip)->pack(-side => 'left');
	$zone_dbox->Button(-text => "Commit change", -command => \&write_changes
)->pack(-side => 'bottom');
foreach $j (@hosts) {
	print "$j\n";
	$tbox->insert("end","$j\n");
	}

}
# Zonefile parser-strips out the Masterfile header, returns a list of IN entries
# I need to modify this to handle NS records (easy) and possibly SOA (not)
sub parse {
my @list;
$zonep= ($zonedir . "/") . $openzone;
open(ZONEFILE, ("<" . $zonep)) || die "error";
$f=0;
foreach $_ (<ZONEFILE>) {
	if ((/(\w.*)[\t ]IN[\t ](\w*)[\t ](.*)/g)) {
		$list[$f] = $_;
		$f++;
	}
}
close ZONEFILE;
return @list;

}
# Save the changes to a zone, making a backup.
# This was a bitch to write
sub write_changes {
$i=0;
$found=0;
open(ZONEFILE, $zonep) || die "error";
my @hosts=<ZONEFILE>;
close(ZONEFILE);
LOOP: while(defined($hosts[$i])) {
	if(is_IN_entry($hosts[$i])) {
	@thisline=split(/[\t ]*/, $hosts[$i]);
	if(($hostname eq $thisline[0]) || ($ip eq $thisline[3])) {
		if ($thisline[2] eq "A") {
			$new_entry="$hostname\t\tIN\tA\t$ip\n";
			$found=1;
			last LOOP;
			}  
		if ($thisline[2] eq "CNAME") {
		$new_entry="$hostname\t\tIN\tCNAME\t$ip\n";
		$found=1;
		last LOOP;
			}
		
	}	if($ip =~ /\d*\..*/) {$new_entry="$hostname\t\tIN\tA\t$ip\n";}
		if( ($hostname =~ /[[:alpha:]].*/) && ($ip =~ /[[:alpha:]]/)) {$new_entry="$hostname\t\tIN\tCNAME\t$ip\n";}
		if($hostname =~ /\d*\..*/) { $new_entry="$ip\t\tIN\tPTR\t$hostname\n";}
	
	}
if(defined($new_entry)) {print "$new_entry\n";}
$i++;	
}
if($found != 1) {$hosts[$i+1]=$new_entry;}
rename($zonep, ($zonep . ".old"));
open(ZONEFILE, (">" . $zonep)) || die "error";
foreach $f (@hosts) {
	if(defined($f)) {print ZONEFILE "$f"};
	}
close(ZONEFILE);
}
# Quick helper sub for write_changes()
sub is_IN_entry {
	$_=$_[0];
	if(/\w*[\t ]IN[\t ]\w*[\t ].*/) {
		return 1;
} else {return 0;}
} 
# This will exit the program.
sub close_program {
	exit;
}

# This is the about section
sub about_tt {
	my $aboutwindow = MainWindow->new;
	$aboutwindow->Label(-text => "TkDNS was started by Sean Lutner (sean\@techtarget.com) mainly to make DNS management easier at work. The backends were written by Adam Glasgall (teferi\@pfnet.org).\n This software falls under the GPL license, and all that stuff.\n\nThis is version 0.2 of this program and therefore will have bugs and stuff, use at your own risk.\n")->pack(-expand => 1, -fill => 'both');

	$aboutwindow->Button(-text => "Close About", 
		-command => sub { $aboutwindow->destroy; })->pack;
} 

# This is the even that occurs when you submit a new zone.
